@@ -8,7 +8,18 @@ use POSIX ();
 use Carp qw(carp croak);
 use English qw(-no_match_vars);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
+
+my $maybe_set_subname = sub { $ARG[1]; };
+
+# Will be shipping with Perl 5.22
+eval {
+    require Sub::Util;
+
+    if (my $set_subname = Sub::Util->can('set_subname')) {
+        $maybe_set_subname = $set_subname;
+    }
+};
 
 my $default_log_callback = sub {
     my ($self, @params) = @ARG;
@@ -28,15 +39,6 @@ my $default_log_callback = sub {
     }
 };
 
-# Will be shipping with Perl 5.22
-eval {
-    require Sub::Util;
-
-    if (my $set_subname = Sub::Util->can('set_subname')) {
-        $set_subname->('default_log_callback', $default_log_callback);
-    }
-};
-
 has $_ => (get => q{+}, set => q{-}) for qw(
     pid
     uid
@@ -47,6 +49,7 @@ has $_ => (get => q{+}, set => q{-}) for qw(
     dry_run
     is_detached
     log_callback
+    on_fork_callback
 );
 
 sub new {
@@ -92,7 +95,7 @@ sub new {
         $logger = $param{'logger'};
 
         croak q{Bad argument "logger"}
-            unless defined $logger and $logger->can('info');
+            unless defined $logger and ref $logger and $logger->can('info');
     }
 
     my $log_callback;
@@ -101,9 +104,21 @@ sub new {
 
         croak q{Bad argument "log_callback"}
             unless defined $log_callback and ref($log_callback) eq 'CODE';
+
+        $log_callback = $maybe_set_subname->('custom_log_callback', $log_callback);
     }
     else {
-        $log_callback = $default_log_callback;
+        $log_callback = $maybe_set_subname->('default_log_callback', $default_log_callback);
+    }
+
+    my $on_fork_callback;
+    if (exists $param{'on_fork_callback'}) {
+        $on_fork_callback = $param{'on_fork_callback'};
+
+        croak q{Bad argument "on_fork_callback"}
+            unless defined $on_fork_callback and ref($on_fork_callback) eq 'CODE';
+
+        $on_fork_callback = $maybe_set_subname->('on_fork_callback', $on_fork_callback);
     }
 
     return bless({}, $class)
@@ -114,6 +129,7 @@ sub new {
         ->_set_umask($umask)
         ->_set_logger($logger)
         ->_set_log_callback($log_callback)
+        ->_set_on_fork_callback($on_fork_callback)
         ->_set_pid($PROCESS_ID)
         ->_set_is_detached(0);
 }
@@ -136,16 +152,20 @@ sub daemonize {
             POSIX::setsid()
                 or croak(sprintf q{Can't start a new session: %s}, $OS_ERROR);
 
-            $self->_set_pid(POSIX::getpid());
             $self->reopen_std;
-            $self->_set_is_detached(1);
+
+            $self->_set_pid(POSIX::getpid())->_set_is_detached(1);
         }
 
         # this branch is the parent
         else {
             $self->_log('Spawned process pid=%d. Parent exiting', $pid);
-            $self->_set_pid($pid);
-            $self->_set_is_detached(1);
+            $self->_set_pid($pid)->_set_is_detached(1);
+
+            if (defined(my $cb = $self->on_fork_callback)) {
+                $cb->($self);
+            }
+
             POSIX::exit(0);
         }
     }
@@ -153,20 +173,18 @@ sub daemonize {
         $self->_set_pid($PROCESS_ID);
     }
 
-    $self->drop_privileges;
-
-    return $self;
+    return $self->drop_privileges;
 }
 
 sub reopen_std {
     my $self = shift;
 
     open STDIN,  '</dev/null'
-        or croak(sprintf q{Can't reopen STDIN: %s},   $OS_ERROR);
+        or croak(sprintf q{Can't reopen STDIN: %s},  $OS_ERROR);
     open STDOUT, '>/dev/null'
-        or croak(sprintf q{Can't reopen STDOUT: %s},  $OS_ERROR);
+        or croak(sprintf q{Can't reopen STDOUT: %s}, $OS_ERROR);
     open STDERR, '>/dev/null'
-        or croak(sprintf q{Can't reopen STDERR: %s},  $OS_ERROR);
+        or croak(sprintf q{Can't reopen STDERR: %s}, $OS_ERROR);
 
     return $self;
 }
@@ -204,10 +222,10 @@ sub drop_privileges {
 sub try_kill {
     my ($self, $signal) = @ARG;
 
-    # parameter order in POSIX.pm
-    # CORE::kill($signal, $pid);
-    # POSIX::kill($pid, $signal);
     if (defined(my $pid = $self->pid)) {
+        # parameter order in POSIX.pm
+        # CORE::kill($signal, $pid);
+        # POSIX::kill($pid, $signal);
         return POSIX::kill($pid, defined $signal ? $signal : q{0});
     }
     else {
@@ -242,7 +260,7 @@ JIP::Daemon - Daemonize server process.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -253,13 +271,15 @@ Just run:
     my $proc = JIP::Daemon->new;
 
     # Send program to backgroung.
-    # If the program is already a running background job, the daemonize method shall have no effect.
-    $proc->daemonize;
+    $proc = $proc->daemonize;
 
     # In the backgroung process:
     $proc->is_detached; # 1
+    $proc->try_kill(0); # 1
     printf qq{pid(%s), is_alive(%d), is_detached(%d)\n}, $proc->status;
-    $proc->try_kill(0);
+
+    # If the program is already a running background job, the daemonize method shall have no effect.
+    $proc = $proc->daemonize;
 
 Dry run:
 
@@ -277,10 +297,22 @@ With logger:
     use Mojo::Log;
     use JIP::Daemon;
 
-    my $proc = JIP::Daemon->new(logger  => Mojo::Log->new);
+    my $proc = JIP::Daemon->new(logger => Mojo::Log->new);
 
     $proc->daemonize;
 
+With on_fork_callback:
+
+    use JIP::Daemon;
+
+    my $proc = JIP::Daemon->new(
+        on_fork_callback => sub {
+            # After daemonizing, and before exiting,
+            # run the given code in parent process
+            print $proc->pid;
+        },
+    )->daemonize;
+
 =head1 ATTRIBUTES
 
 L<JIP::Daemon> implements the following attributes.
@@ -354,6 +386,10 @@ With custom callback:
     );
     $proc->log_callback->($proc, 'Hello');
 
+=head2 on_fork_callback
+
+After daemonizing, and before exiting, run the given code in parent process.
+
 =head1 METHODS
 
 =head2 new
@@ -396,7 +432,7 @@ Returns a list of process attributes: PID, is alive, is detached (in backgroung)
 
 =head1 SEE ALSO
 
-POSIX, Privileges::Drop, Object::ForkAware, Proc::Daemon
+POSIX, Privileges::Drop, Object::ForkAware, Proc::Daemon, Daemon::Daemonize
 
 =head1 AUTHOR
 
